@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.23 2004/12/06 11:00:09 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.24 2004/12/14 05:09:29 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -83,6 +83,8 @@ long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 
 static int stop_sig = SIGTERM;
 
+static int degenerated = 0;	/* set non 0 if already degerated */
+
 /*
 * pgpool main program
 */
@@ -184,6 +186,9 @@ int main(int argc, char **argv)
 	}
 
 
+	/* set signal masks */
+	poolinitmask();
+
 	if (not_detach)
 		write_pid_file();
 	else
@@ -197,7 +202,7 @@ int main(int argc, char **argv)
 			 pool_config.port);
 
 	/* set up signal handlers */
-	signal(SIGPIPE, SIG_IGN);
+	pool_signal(SIGPIPE, SIG_IGN);
 
 	/* create unix domain socket */
 	unix_fd = create_unix_domain_socket();
@@ -223,16 +228,35 @@ int main(int argc, char **argv)
 	}
 
 	/* set up signal handlers */
-	signal(SIGTERM, exit_handler);
-	signal(SIGINT, exit_handler);
-	signal(SIGQUIT, exit_handler);
-	signal(SIGCHLD, reap_handler);
-	signal(SIGUSR1, failover_handler);
-	signal(SIGUSR2, failover_handler);
+	pool_signal(SIGTERM, exit_handler);
+	pool_signal(SIGINT, exit_handler);
+	pool_signal(SIGQUIT, exit_handler);
+	pool_signal(SIGCHLD, reap_handler);
+	pool_signal(SIGUSR1, failover_handler);
+	pool_signal(SIGUSR2, failover_handler);
 
 	for (;;)
 	{
+#ifdef HEALTH_CHECK
+		int sts;
+
+		sleep(5);
+
+		if (!degenerated)
+		{
+			sts = health_check();
+			if (sts == -1)
+			{
+				failover_handler(SIGUSR1);
+			}
+			else if (sts == -2)
+			{
+				failover_handler(SIGUSR2);
+			}
+		}
+#else
 		pause();
+#endif
 	}
 	return 0;
 }
@@ -391,6 +415,7 @@ pid_t fork_a_child(int unix_fd, int inet_fd)
 	if (pid == 0)
 	{
 		/* call child main */
+		POOL_SETMASK(&UnBlockSig);
 		do_child(unix_fd, inet_fd);
 	}
 	else if (pid == -1)
@@ -532,13 +557,17 @@ void notice_backend_error(int master)
 		kill(parent, SIGUSR2);
 
 	/* avoid race conditon with SIGCHLD */
+#ifdef NOT_USED
 	sleep(1);
+#endif
 }
 
 static RETSIGTYPE exit_handler(int sig)
 {
 	int i;
 
+	POOL_SETMASK(&BlockSig);
+
 	/*
 	 * this could happend in a child process if a signal has been sent
 	 * before resetting signal handler
@@ -546,6 +575,7 @@ static RETSIGTYPE exit_handler(int sig)
 	if (getpid() != mypid)
 	{
 		pool_debug("exit_handler: I am not parent");
+		POOL_SETMASK(&UnBlockSig);
 		exit(0);
 	}
 
@@ -567,6 +597,8 @@ static RETSIGTYPE exit_handler(int sig)
 	if (errno != ECHILD)
 		pool_error("wait() failed. reason:%s", strerror(errno));
 
+	POOL_SETMASK(&UnBlockSig);
+
 	myexit(0);
 }
 
@@ -581,25 +613,33 @@ static RETSIGTYPE failover_handler(int sig)
 {
 	int i;
 	int replication = 0;
-	static int degenerated = 0;	/* set non 0 if already degerated */
+
+	POOL_SETMASK(&BlockSig);
 
 	pool_debug("failover_handler called");
 
 	/*
-	 * this could happend in a child process if a signal has been sent
+	 * this could happen in a child process if a signal has been sent
 	 * before resetting signal handler
 	 */
 	if (getpid() != mypid)
 	{
-		pool_debug("exit_handler: I am not parent");
+		pool_debug("failover_handler: I am not parent");
+		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
 	if (exiting)
+	{
+		POOL_SETMASK(&UnBlockSig);
 		return;
+	}
 
 	if (switching)
+	{
+		POOL_SETMASK(&UnBlockSig);
 		return;
+	}
 
 #ifdef NOT_USED
 	/* secondary backend exists? */
@@ -607,8 +647,12 @@ static RETSIGTYPE failover_handler(int sig)
 		return;
 #endif
 
-	/* already failover */
-	if (strcmp(pool_config.current_backend_host_name, pool_config.secondary_backend_host_name) ||
+	/* 
+	 * if not in replication mode, we treat this a restart request.
+	 * otherwise we need to check if we have already failovered.
+	 */
+	if (!pool_config.replication_enabled ||
+		strcmp(pool_config.current_backend_host_name, pool_config.secondary_backend_host_name) ||
 		pool_config.current_backend_port != pool_config.secondary_backend_port)
 	{
 		switching = 1;
@@ -654,6 +698,7 @@ static RETSIGTYPE failover_handler(int sig)
 				pool_debug("kill %d", pid);
 			}
 		}
+
 		while (wait(NULL) > 0)
 			;
 
@@ -719,6 +764,7 @@ static RETSIGTYPE failover_handler(int sig)
 
 		switching = 0;
 	}
+	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -730,13 +776,21 @@ static RETSIGTYPE reap_handler(int sig)
 	int status;
 	int i;
 
+	POOL_SETMASK(&BlockSig);
+
 	pool_debug("reap_handler called");
 
 	if (exiting)
+	{
+		POOL_SETMASK(&UnBlockSig);
 		return;
+	}
 
 	if (switching)
+	{
+		POOL_SETMASK(&UnBlockSig);
 		return;
+	}
 
 #ifdef HAVE_WAITPID
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
@@ -763,4 +817,6 @@ static RETSIGTYPE reap_handler(int sig)
 				}
 			}
 		}
+		POOL_SETMASK(&UnBlockSig);
+
 	}
