@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /home/t-ishii/repository/pgpool/pool.h,v 1.26 2004/11/12 11:45:40 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool.h,v 1.27 2004/12/14 05:09:29 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -27,6 +27,7 @@
 #define POOL_H
 
 #include "config.h"
+#include "pool_signal.h"
 #include <stdio.h>
 #include <time.h>
 
@@ -304,5 +305,6 @@ extern void pool_send_error_message(POOL_CONNECTION *frontend, int protoMajor,
 							 int line);
 
 extern void pool_free_startup_packet(StartupPacket *sp);
+extern int health_check(void);
 
 #endif /* POOL_H */
