@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.26 2004/11/12 11:45:40 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.27 2004/12/14 05:09:29 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -215,7 +215,7 @@ void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend)
 
 	/* no other timer found. set my timer */
 	pool_debug("pool_connection_pool_timer: set alarm after %d seconds", pool_config.connection_life_time);
-	signal(SIGALRM, pool_backend_timer_handler);
+	pool_signal(SIGALRM, pool_backend_timer_handler);
 	alarm(pool_config.connection_life_time);
 }
 
@@ -231,6 +231,8 @@ RETSIGTYPE pool_backend_timer_handler(int sig)
 	time_t now;
 	time_t nearest = TMINTMAX;
 
+	POOL_SETMASK(&BlockSig);
+
 	now = time(NULL);
 
 	pool_debug("pool_backend_timer_handler called at %d", now);
@@ -281,9 +283,11 @@ RETSIGTYPE pool_backend_timer_handler(int sig)
 		nearest = pool_config.connection_life_time - (now - nearest);
 		if (nearest <= 0)
 		  nearest = 1;
-		signal(SIGALRM, pool_backend_timer_handler);
+		pool_signal(SIGALRM, pool_backend_timer_handler);
 		alarm(nearest);
 	}
+
+	POOL_SETMASK(&UnBlockSig);
 }
 
 int connect_inet_domain_socket(int secondary_backend)
@@ -418,7 +422,7 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 	}
 	create_cp(MASTER_CONNECTION(p), 0);
 
-			/* initialize Paramter Status save structure */
+	/* initialize Paramter Status save structure */
 	if (pool_init_params(&MASTER(p)->params))
 	{
 		return NULL;
